@@ -1,7 +1,13 @@
 // Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
 
+use std::borrow::Cow;
 use std::fmt;
 
+use crate::swc::common::Span;
+use crate::swc::common::Spanned;
+use crate::swc::parser::error::SyntaxError;
+use crate::SourceTextInfo;
+
 /// A 0-indexed line and column type.
 pub type LineAndColumnIndex = text_lines::LineAndColumnIndex;
 
@@ -14,10 +20,34 @@ pub type LineAndColumnDisplay = text_lines::LineAndColumnDisplay;
 pub struct Diagnostic {
   /// Specifier of the source the diagnostic occurred in.
   pub specifier: String,
+  /// Range of the diagnostic.
+  pub span: Span,
   /// 1-indexed display position the diagnostic occurred at.
   pub display_position: LineAndColumnDisplay,
+  /// Swc syntax error
+  pub kind: SyntaxError,
+}
+
+impl Diagnostic {
   /// Message text of the diagnostic.
-  pub message: String,
+  pub fn message(&self) -> Cow<str> {
+    self.kind.msg()
+  }
+}
+
+impl Diagnostic {
+  pub(crate) fn from_swc_error(
+    err: crate::swc::parser::error::Error,
+    specifier: &str,
+    source: &SourceTextInfo,
+  ) -> Diagnostic {
+    Diagnostic {
+      span: err.span(),
+      display_position: source.line_and_column_display(err.span().lo),
+      specifier: specifier.to_string(),
+      kind: err.into_kind(),
+    }
+  }
 }
 
 impl std::error::Error for Diagnostic {}
@@ -27,7 +57,7 @@ impl fmt::Display for Diagnostic {
     write!(
       f,
       "{} at {}:{}:{}",
-      self.message,
+      self.message(),
       self.specifier,
       self.display_position.line_number,
       self.display_position.column_number
